c ******************************************************* c
c                                                         c
c     A term for cross power spectrum in TNS model        c
c                                                         c       
c     Version based on SPT 1-loop order                   c                     
c                                                         c 
c ******************************************************* c
c
c     Note--. 
c
c     All the parameter 'ikmax' in this program must be 
c     the same. 
c
c     Exponential cutoff is taken into account 
c     (see function integ_fp). 
c
      program A_term_cross
c
      implicit none
c
      integer  ik, ikmax, ik_max, ik_max1, ik_max2
      integer  itype, isigmav
      parameter(ikmax=3000)
      real*8  ak(ikmax), pk(ikmax)
      real*8  akcorr(ikmax)
      real*8   pi, zred, ss, sigmav, sigma_v, growth
      character infile*50
      common /pk_data/ ak, pk, ik_max
      pi = 4.d0 * atan(1.d0)
c     -------------------------------------------------
c
      write(6,*)
      write(6,*) '*** Corrections to Scoccimarro (2004) formula ***'
      write(6,*) '*** (A term) in redshift-space power spectrum ***'
      write(6,*)
      write(6,*)
      write(6,*) 'Output redshift ?'
      read(5,*) zred
c
c     /////// Load (linear) matter power spectrum ///////
c
      call load_matterpower_data
c
      write(6,*) ' loading (linear) matter power spectrum, done '
c
c     //////// Set cosmological parameters ////////
c
      call set_cosmological_params
c
c     /////// Sigma8 normalization ///////
c
      call normalization_trapez
c
      write(6,*) ' normalization done '
c
c     //////// Truncation of low-k, high-k  in linear P(k) ////////
c
      call truncation_k
c
c     /////// Correction to Scoccimarro (2004) formula ///////
c
      call calc_corrections(zred)
c
      write(6,*) ' 1-loop P(k) done '
c
c     /////// Summing up all contributions ///////
c
      call calc_pkred(zred,sigmav)
c
      write(6,*) ' Computing A term, done '
c
c
      end
c
c ******************************************************* c
c
      subroutine load_matterpower_data
c
c ******************************************************* c
c
c     input file is assumed to be the matter power spectrum data 
c     created by CAMB code. 
c
      implicit none
c
      integer ik, ikk, ikmax, ik_max
      parameter(ikmax=3000)
      real*8  ak(ikmax), pk(ikmax), pi
      common /pk_data/ ak, pk, ik_max
      character infile*50
      pi = 4.d0 * datan(1.d0)
c     --------------------------------------------------------
c
      write(6,*) ' Type file name of matter P(k) data'
      read(5,*) infile
      open(9, file=infile, status='unknown')
c
      do ik=1, ikmax
         read(9,*,END=10) ak(ik), pk(ik)
      enddo
c
 10   continue
      ik_max = ik - 1
c
      write(6,*) 'ak(1)=', ak(1)
      write(6,*) 'ak(ik_max)=', ak(ik_max)
      write(6,*) 'ik_max=', ik_max
      write(6,*)
c
      close(9)
c
      end
c
c ******************************************************* c
c
      subroutine set_cosmological_params
c
c ******************************************************* c
c
c     Set cosmological parameters for linear growth factor,
c     growth-rate parameter, and sigma8 normalization 
c
c     Note.--
c     assume a flat cosmological model (Omega_v = 1-Omega_m)
c
      implicit none
      integer iparams
      real*8 sigma8, Omega_m, Omega_v, w_de
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
c     ---------------------------------------------------
c
      write(6,*)
      write(6,*) '*** Set cosmological parameters ***' 
      write(6,*)
c
c     Default cosmological parameters (you can change them appropriately)
c
      sigma8 = 0.801d0
      Omega_m = 0.2573d0
      Omega_v = 1.d0 - Omega_m
      w_de = -1.d0
c
 5    write(6,*) '[1] sigma_8  =',sigma8
      write(6,*) '[2] Omega_m  =',Omega_m
      write(6,*) '[3] w_de     =',w_de
      write(6,*) 
      write(6,*) 'Note--. spatial curvature is forced to be flat, '
      write(6,*) '                   i.e., Omega_v = 1 - Omega_m  '
      write(6,*)
      write(6,*) 'change cosmological parameter? [1-7] or n[0]'
      read(5,*)  iparams
      if(iparams.eq.0) goto 8
      if(iparams.eq.1) then
         write(6,*) 'type sigma8'
         read(5,*) sigma8
      elseif(iparams.eq.2) then
         write(6,*) 'type Omega_m'
         read(5,*) Omega_m
      elseif(iparams.eq.3) then
         write(6,*) 'type w'
         read(5,*) w_de
      else 
         stop
      endif
      goto 5
c
 8    continue
c      
      end
c
c ******************************************************* c
c
      subroutine normalization_trapez
c
c ******************************************************* c
c
      implicit none
      integer ik, ik_max, ikmax
      parameter(ikmax=3000)
      real*8  Omega_m, Omega_v, w_de, sigma8
      real*8  ak(ikmax), pk(ikmax)
      real*8  r_th, const, x
      real*8  W_TH, sigma_a, sigma_b, pi
      common /pk_data/ ak, pk, ik_max
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
      pi = 4.d0 * atan(1.d0)
      r_th = 8.d0
c     ---------------------------------------------------
c
      x = ak(1) * r_th
      if(x.lt.1.d-3) then
         W_TH = 1.d0 - x*x / 10.d0 + x**4 / 280.d0 
      else
         W_TH = 3.d0 * (sin(x) - x * cos(x))/x/x/x
      endif
      sigma_a = W_TH * W_TH * pk(1) * ak(1) * ak(1)
      sigma_a = sigma_a / (2.d0 * pi * pi)
c
      const = 0.d0 
      do ik=2, ik_max
         x = ak(ik) * r_th
         if(x.lt.1.d-3) then
            W_TH = 1.d0 - x*x / 10.d0 + x**4 / 280.d0 
         else
            W_TH = 3.d0 * (sin(x) - x * cos(x))/x/x/x
         endif
         sigma_b = W_TH * W_TH * pk(ik) * ak(ik) * ak(ik) 
         sigma_b = sigma_b / (2.d0 * pi * pi)
         const = const + 
     &        (sigma_a + sigma_b) * ( ak(ik) - ak(ik-1) )/ 2.d0
         sigma_a = sigma_b
      enddo
c
      do ik=1, ik_max
         pk(ik) = sigma8 * sigma8 / const * pk(ik) 
      enddo
c
      end
c
c ******************************************************* c
c
      subroutine truncation_k
c
c ******************************************************* c
c
      implicit none
c
      integer ik, ikk, ikmax, ik_max
      parameter(ikmax=3000)
      real*8 ak(ikmax), pk(ikmax)
      real*8 akk(ikmax), pkk(ikmax)
      real*8 kmin, kmax, pi
      common /pk_data/ ak, pk, ik_max
      common /kmin_kmax/ kmin, kmax
      pi = 4.d0 * datan(1.d0)
c     -----------------------------------------------------
c
      kmin = 5.d-4   ! default value
      kmax = 10.d0    ! default value
c
      do ik=1, ik_max
         akk(ik) = ak(ik)
         pkk(ik) = pk(ik)
      enddo
c
      ikk = 1
      do ik=1, ik_max
         if(akk(ik).ge.kmin .and. akk(ik).le.kmax) then
cc     &        .and. mod(ik,2).eq.0 ) then
            ak(ikk) = akk(ik)
            pk(ikk) = pkk(ik)
            ikk = ikk + 1
         endif
      enddo
c
      ik_max = ikk -1
c
      write(6,*) 'ak(1)=', ak(1)
      write(6,*) 'ak(ik_max)=', ak(ik_max)
      write(6,*) 'ik_max=', ik_max
      write(6,*)
c
      end
c
c ******************************************************* c
c
      subroutine find_pk(kk, pklin)
c
c ******************************************************* c
c
      implicit none
      integer ik_max, ikmax
      integer j, jmin, jmax
      parameter(ikmax=3000)
      real*8 ak(ikmax), pk(ikmax), kk, s, ds, pklin, kmin, kmax
      common /pk_data/ ak, pk, ik_max
      common /kmin_kmax/ kmin, kmax      
c     -------------------------------------------
c
      if(kk.ge.kmin .and. kk.le.kmax) then
c
         call hunt(ak, ik_max, kk, j)
c
         jmin = j - 2
         jmax = j + 2
         if(jmin.lt.1) jmin = 1
         if(jmax.ge.ik_max) jmax = ik_max
c     
         call polint(ak(jmin),pk(jmin),jmax-jmin+1,kk,s,ds)
         pklin = s
      else
         pklin = 0.d0
      endif
c      
      end
c
c ******************************************************* c
c
      subroutine calc_corrections(zred)
c
c ******************************************************* c
c
      implicit none
      integer ik, ik_max, ikmax, isub, ip
      integer ix, ixmax
      parameter(ikmax=3000)
      parameter(ixmax=300)
      real*8  pi
      real*8  ak(ikmax), pk(ikmax)
      real*8  pk_A111(ikmax), pk_A121(ikmax), pk_A212(ikmax)
      real*8  pk_A221(ikmax), pk_A222(ikmax), pk_A322(ikmax)
      real*8  pk_tA111(ikmax), pk_tA222(ikmax), pk_tA322(ikmax)
      real*8  pk_hA112(ikmax), pk_hA212(ikmax), pk_hA221(ikmax)
      real*8  kmin, kmax, xmin, xmax, mumin, mumax
      real*8  k, ww(ixmax), xx(ixmax)
      real*8  int_Aa(3), int_As(3), int_tA(3), int_hA(3), zred, zz
      common /redshift/ zz 
      common /pk_data/ ak, pk, ik_max
      common /wave_number/  k, xmin, xmax
      common /corr_pk/ pk_A111, pk_A121, pk_A212, pk_A221, pk_A222, 
     &     pk_A322, 
     &     pk_tA111, pk_tA222, pk_tA322, 
     &     pk_hA112, pk_hA212, pk_hA221
      pi = 4.d0 * datan(1.d0)
c     ---------------------------------------------------
c
      zz = zred
      kmin = ak(1)
      kmax = ak(ik_max) 
c
      do 10 ik=1, ik_max
c
         k = ak(ik)
         pk_A111(ik)=0.d0 
         pk_A121(ik)=0.d0
         pk_A212(ik)=0.d0
         pk_A221(ik)=0.d0
         pk_A222(ik)=0.d0
         pk_A322(ik)=0.d0
         pk_tA111(ik)=0.d0 
         pk_tA222(ik)=0.d0
         pk_tA322(ik)=0.d0
         pk_hA112(ik)=0.d0
         pk_hA212(ik)=0.d0
         pk_hA221(ik)=0.d0
c
         xmin = kmin / k
         xmax = kmax / k
c
c     ////// Gauss-Legendre integration //////  c
c
cc         if(k.lt.0.2) isub =200 
cc         if(k.ge.0.2) isub =0 
         isub = 0
c
         call gauleg(log(xmin),log(xmax),xx,ww,ixmax-isub)
c
         do ix=1, ixmax-isub
            xx(ix)= dexp(xx(ix))
            call integ_fp_sym(xx(ix),  int_As, int_tA)
            call integ_fp_asym(xx(ix), int_Aa, int_hA)
            pk_A111(ik) = pk_A111(ik) +ww(ix)*int_As(1)
            pk_A222(ik) = pk_A222(ik) +ww(ix)*int_As(2)
            pk_A322(ik) = pk_A322(ik) +ww(ix)*int_As(3)
            pk_tA111(ik)= pk_tA111(ik)+ww(ix)*int_tA(1)
            pk_tA222(ik)= pk_tA222(ik)+ww(ix)*int_tA(2)
            pk_tA322(ik)= pk_tA322(ik)+ww(ix)*int_tA(3)
c            
            pk_A121(ik) = pk_A121(ik) +ww(ix)*int_Aa(1)
            pk_A212(ik) = pk_A212(ik) +ww(ix)*int_Aa(2)
            pk_A221(ik) = pk_A221(ik) +ww(ix)*int_Aa(3)
            pk_hA112(ik)= pk_hA112(ik)+ww(ix)*int_hA(1)
            pk_hA212(ik)= pk_hA212(ik)+ww(ix)*int_hA(2)
            pk_hA221(ik)= pk_hA221(ik)+ww(ix)*int_hA(3)
         enddo
c
         pk_A111(ik)  = 2.d0 * pk_A111(ik)  * k**3 / (2.*pi)**2
         pk_A222(ik)  = 2.d0 * pk_A222(ik)  * k**3 / (2.*pi)**2
         pk_A322(ik)  = 2.d0 * pk_A322(ik)  * k**3 / (2.*pi)**2
         pk_tA111(ik) = 2.d0 * pk_tA111(ik) * k**3 / (2.*pi)**2
         pk_tA222(ik) = 2.d0 * pk_tA222(ik) * k**3 / (2.*pi)**2
         pk_tA322(ik) = 2.d0 * pk_tA322(ik) * k**3 / (2.*pi)**2
c
         pk_A121(ik)  = pk_A121(ik)  * k**3 / (2.*pi)**2
         pk_A212(ik)  = pk_A212(ik)  * k**3 / (2.*pi)**2
         pk_A221(ik)  = pk_A221(ik)  * k**3 / (2.*pi)**2         
         pk_hA112(ik) = pk_hA112(ik) * k**3 / (2.*pi)**2
         pk_hA212(ik) = pk_hA212(ik) * k**3 / (2.*pi)**2
         pk_hA221(ik) = pk_hA221(ik) * k**3 / (2.*pi)**2
c
c
      write(6,'(i3,1p13e18.10)') ik,k, pk_A111(ik), pk_A222(ik),
     &        pk_A322(ik), pk_tA111(ik), pk_tA222(ik), pk_tA322(ik), 
     &        pk_A121(ik), pk_A212(ik), pk_A221(ik),
     &        pk_hA112(ik), pk_hA212(ik), pk_hA221(ik)
c
 10   continue
c
      end
c
c ******************************************************* c
c
      subroutine integ_fp_sym(x, int_As, int_tA)
c
c ******************************************************* c
c
c     int_As(1):  A111,     int_tA(1):  tA111
c     int_As(2):  A222,     int_tA(2):  tA222
c     int_As(3):  A322,     int_tA(3):  tA322
c
      implicit none
      integer ip, imu, imu_max
      parameter(imu_max=20)
      real*8  int_As(3), int_tA(3), xmin, xmax, mumin, mumax
      real*8  integ_fp_As(3), integ_fp_tA(3)
      real*8  k, x, wmu(imu_max), mmu(imu_max)
      real*8  p, kp
      common /wave_number/  k, xmin, xmax
c     ------------------------------------------  c
c
      mumin = max(-1.0, (1.+x**2-xmax**2)/2./x)
      mumax = min( 1.0, (1.+x**2-xmin**2)/2./x)
c
      if(x.ge.0.5d0) mumax= 0.5d0/x
c
      call gauleg(mumin, mumax, mmu, wmu, imu_max)
c
      do ip=1, 3
         int_As(ip) = 0.d0
         int_tA(ip) = 0.d0
      enddo
c
      do imu=1, imu_max
c
         p = k * x
c
         call fp_As(x, mmu(imu), integ_fp_As, integ_fp_tA)
c
         do ip=1, 3
            int_As(ip) = int_As(ip) + wmu(imu) * integ_fp_As(ip)
            int_tA(ip) = int_tA(ip) + wmu(imu) * integ_fp_tA(ip)
         enddo
c
      enddo
c
 10   continue
c
      end
c
c ******************************************************* c
c
      subroutine integ_fp_asym(x, int_Aa, int_hA)
c
c ******************************************************* c
c
c     int_Aa(1):  A121,     int_hA(1):  hA112
c     int_Aa(2):  A212,     int_hA(2):  hA222
c     int_Aa(3):  A221,     int_hA(3):  hA221
c
      implicit none
      integer ip, imu, imu_max
      parameter(imu_max=30)
      real*8  int_Aa(3), int_hA(3), xmin, xmax, mumin, mumax
      real*8  integ_fp_Aa(3), integ_fp_hA(3)
      real*8  k, x, wmu(imu_max), mmu(imu_max)
      real*8  p, kp
      common /wave_number/  k, xmin, xmax
c     ------------------------------------------  c
c
      mumin = max(-1.0, (1.+x**2-xmax**2)/2./x)
      mumax = min( 1.0, (1.+x**2-xmin**2)/2./x)
c
      call gauleg(mumin, mumax, mmu, wmu, imu_max)
c
      do ip=1, 3
         int_Aa(ip) = 0.d0
         int_hA(ip) = 0.d0
      enddo
c
      do imu=1, imu_max
c
         p = k * x
c
         call fp_Aa(x, mmu(imu), integ_fp_Aa, integ_fp_hA)
c
         do ip=1, 3
            int_Aa(ip) = int_Aa(ip) + wmu(imu) * integ_fp_Aa(ip)
            int_hA(ip) = int_hA(ip) + wmu(imu) * integ_fp_hA(ip)
         enddo
c
      enddo
c
 10   continue
c
      end
c
c ******************************************************* c
c
      subroutine  fp_As(x, mu, integ_fp_As, integ_fp_tA)
c
c ******************************************************* c
c
c     integ_fp_As(1):  A111,    integ_fp_tA(1):  tA111
c     integ_fp_As(2):  A222,    integ_fp_tA(2):  tA222
c     integ_fp_As(3):  A322,    integ_fp_tA(3):  tA322
c
      implicit none
      real*8 mu, k, x
      real*8 xmax, xmin, integ_fp_As(3), integ_fp_tA(3)
      real*8 p, kp, zred
      real*8 b211A, b222A, b211tA, b222tA
      common /redshift/ zred
      common /wave_number/  k, xmin, xmax
c     --------------------------------------- c
c
      p = k * x
      kp = k * dsqrt(1.d0 + x*x - 2.d0*x*mu)
c
      integ_fp_As(1) = x * mu 
      integ_fp_As(2) = - x*x * (3.*x*mu-2.) * (mu*mu-1.) 
     &     / (1.+x*x-2.*mu*x) /2.
      integ_fp_As(3) = x*( 2.*mu + x*( 2. - 6.*mu*mu + 
     &     x*mu*(-3.+5.*mu*mu) ) ) / (1.+x*x-2.*mu*x) / 2.
c
      integ_fp_tA(1) = - x*x * (x*mu-1.) / (1.+x*x-2.*x*mu) 
      integ_fp_tA(2) = x*x * (3.*x*mu-1.) * (mu*mu-1.)
     &     / (1.+x*x-2.*x*mu) /2.
      integ_fp_tA(3) = x*x * (-1. + 3.*x*mu + 3.*mu*mu - 5.*x*mu**3 ) 
     &     / (1.+x*x-2.*x*mu) /2.
c
      call Bispec1(zred, p, kp, k, b211A, b222A, b211tA, b222tA)
c
      integ_fp_As(1) = integ_fp_As(1) * b211A * x
      integ_fp_As(2) = integ_fp_As(2) * b222A * x
      integ_fp_As(3) = integ_fp_As(3) * b222A * x
c
      integ_fp_tA(1) = integ_fp_tA(1) * b211tA * x
      integ_fp_tA(2) = integ_fp_tA(2) * b222tA * x
      integ_fp_tA(3) = integ_fp_tA(3) * b222tA * x
c
      end
c
c ******************************************************* c
c
      subroutine  fp_Aa(x, mu, integ_fp_Aa, integ_fp_hA)
c
c ******************************************************* c
c
c     int_fp_Aa(1):  A121,     int_fp_hA(1):  hA112
c     int_fp_Aa(2):  A212,     int_fp_hA(2):  hA212
c     int_fp_Aa(3):  A221,     int_fp_hA(3):  hA221
c
      implicit none
      real*8 mu, k, x
      real*8 xmax, xmin, integ_fp_Aa(3), integ_fp_hA(3)
      real*8 p, kp, zred
      real*8 b221A, b212A, b221hA, b212hA
      common /redshift/ zred
      common /wave_number/  k, xmin, xmax
c     --------------------------------------- c
c
      p = k * x
      kp = k * dsqrt(1.d0 + x*x - 2.d0*x*mu)
c
      integ_fp_Aa(1) = - x*x * (3.*x*mu-2.) * (mu*mu-1.) 
     &     / (1.+x*x-2.*mu*x) /2.
      integ_fp_Aa(2) = x * mu
      integ_fp_Aa(3) = x*( 2.*mu + x*(2.-6.*mu*mu) + 
     &     x**2*mu*(-3.+5.*mu*mu) ) / (1.+x*x-2.*mu*x) / 2.
c
      integ_fp_hA(1) =   x*x * (3.*x*mu-1.) * (mu*mu-1.) 
     &     / (1.+x*x-2.*mu*x) /2.
      integ_fp_hA(2) = x*x * (-1. + 3.*x*mu + 3.*mu*mu - 5.*x*mu**3 ) 
     &     / (1.+x*x-2.*x*mu) /2.
      integ_fp_hA(3) = - x*x * (x*mu-1.) / (1.+x*x-2.*x*mu)
c      
      call Bispec2(zred, p, kp, k, b221A, b212A, b221hA, b212hA)
c
      integ_fp_Aa(1) = integ_fp_Aa(1) * b221A * x
      integ_fp_Aa(2) = integ_fp_Aa(2) * b212A * x
      integ_fp_Aa(3) = integ_fp_Aa(3) * b221A * x
c
      integ_fp_hA(1) = integ_fp_hA(1) * b212hA * x
      integ_fp_hA(2) = integ_fp_hA(2) * b212hA * x
      integ_fp_hA(3) = integ_fp_hA(3) * b221hA * x
c
      end      
c
c ******************************************************* c
c
      subroutine calc_pkred(zred)
c
c ******************************************************* c
c
c     This subroutine gives you the coefficients for A term
c     expanded in powers of mu and f(z)
c
c     A(k,mu) = mu**2*( f*    (pk_A11a * bA*bB*cA + pk_A11b * bA*bB*cB) 
c                     + f**2* (pk_A12a * bB*cA**2 + pk_A12b * bA*cB**2) )
c             + mu**4*( f**2* (pk_A22a * bB*cA**2 + pk_A22b * bA*cA*cB
c                            + pk_A22c * bB*cA*cB + pk_A22d * bA*cB**2 ) 
c                     + f**3* (pk_A23a * cA**2*cB + pk_A23b * cA*cB**2) ) 
c             + mu**6*( f**3* (pk_A33a * cA**2*cB + pk_A33b * cA*cB**2) )
c
c      pk_A11a:    pk_A111(ik),  ! mu^2 * f     * bA*bB*cA
c      pk_A11b:    pk_tA111(ik), ! mu^2 * f     * bA*bB*cB
c      pk_A12a:    pk_A121(ik),  ! mu^2 * f*f   * bB*cA**2
c      pk_A12b:    pk_hA112(ik), ! mu^2 * f*f   * bA*cB**2        
c      pk_A22a:    pk_A221(ik),  ! mu^4 * f*f   * bB*cA**2
c      pk_A22b:    pk_A212(ik),  ! mu^4 * f*f   * bA*cA*cB
c      pk_A22c:    pk_hA221(ik), ! mu^4 * f*f   * bB*cA*cB
c      pk_A22d:    pk_hA212(ik), ! mu^4 * f*f   * bA*cB**2 
c      pk_A23a:    pk_A222(ik),  ! mu^4 * f*f*f * cA**2*cB
c      pk_A23b:    pk_tA222(ik), ! mu^4 * f*f*f * cA*cB**2
c      pk_A33a:    pk_A322(ik),  ! mu^6 * f*f*f * cA**2*cB
c      pk_A33b:    pk_tA322(ik), ! mu^6 * f*f*f * cA*cB**2
c     
      implicit none
      integer ik, ikmax, ik_max, ik_max1, ik_max2
      parameter(ikmax=3000)
      real*8  zred, growth, f, dd2, ff
      real*8  Omega_m, Omega_v, w_de, sigma8
      real*8  ak(ikmax), pk(ikmax)
      real*8  akcorr(ikmax)
      real*8  pk_A111(ikmax), pk_A121(ikmax), pk_A212(ikmax)
      real*8  pk_A221(ikmax), pk_A222(ikmax), pk_A322(ikmax)
      real*8  pk_tA111(ikmax), pk_tA222(ikmax), pk_tA322(ikmax)
      real*8  pk_hA112(ikmax), pk_hA212(ikmax), pk_hA221(ikmax)
c
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
      common /pk_data/ ak, pk, ik_max
      common /corr_pk/ pk_A111, pk_A121, pk_A212, pk_A221, pk_A222, 
     &     pk_A322, 
     &     pk_tA111, pk_tA222, pk_tA322, 
     &     pk_hA112, pk_hA212, pk_hA221
c     -------------------------------------------------------- c
      ik_max1 = ik_max
      ik_max2 = ik_max
c
c     growth factor and its logarithmic derivative
c     (assuming flat universe)
      dd2 = growth(zred)**2
      ff = f(zred)
c
      write(6,'(A,1p2e18.10)') 'd, f=', dd2, ff
c
c     ////// corrections to power spectrum  ////// c
c
      open(11,file='pkcross_Aterm.dat',status='unknown')
c
      do ik=1, ik_max1
c
         akcorr(ik) = ak(ik)
c
         write(11,'(1p13e18.10)') akcorr(ik), 
     &        pk_A111(ik),  ! mu^2 * f     * bA*bB*cA
     &        pk_tA111(ik), ! mu^2 * f     * bA*bB*cB
     &        pk_A121(ik),  ! mu^2 * f*f   * bB*cA**2
     &        pk_hA112(ik), ! mu^2 * f*f   * bA*cB**2        
     &        pk_A221(ik),  ! mu^4 * f*f   * bB*cA**2
     &        pk_A212(ik),  ! mu^4 * f*f   * bA*cA*cB
     &        pk_hA221(ik), ! mu^4 * f*f   * bB*cA*cB
     &        pk_hA212(ik), ! mu^4 * f*f   * bA*cB**2 
     &        pk_A222(ik),  ! mu^4 * f*f*f * cA**2*cB
     &        pk_tA222(ik), ! mu^4 * f*f*f * cA*cB**2
     &        pk_A322(ik),  ! mu^6 * f*f*f * cA**2*cB
     &        pk_tA322(ik)  ! mu^6 * f*f*f * cA*cB**2
      enddo
c
      close(11)
      write(6,*) '> Output file: pkcross_Aterm.dat '
c
      end
c
c ******************************************************* c
c
      subroutine Bispec1(zred, p, kp, k, b211A, b222A, b211tA, b222tA)
c                                                         c
c ******************************************************* c
c
c     bispectrum at tree-level SPT (kp = k-p)
c
c      b211A = Bispec(2, 1, 1, p, kp, -k, zred)
c      b222A = Bispec(2, 2, 2, p, kp, -k, zred)
c
c      b211tA = Bispec(2, 1, 1, kp, p, -k, zred)
c      b222tA = Bispec(2, 2, 2, kp, p, -k, zred)
c
      implicit none
c
      real*8  zred, k, p, kp
      real*8  b211A, b222A, b211tA, b222tA
      real*8  dd2, growth, pklin_k, pklin_p, pklin_kp
      real*8  F2_1_kp_k, F2_2_kp_k, F2_1_p_k
      real*8  F2_2_p_k, F2_1_p_kp, F2_2_p_kp
c     -----------------------------------------   c
c
      dd2 = growth(zred)**2
c
      call find_pk(k, pklin_k)
      call find_pk(p, pklin_p)
      call find_pk(kp, pklin_kp)
c
c     bispectrum at tree-level order
c
      call F2_tree(k, p, kp, 
     &     F2_1_kp_k, F2_1_p_k, F2_1_p_kp, 
     &     F2_2_kp_k, F2_2_p_k, F2_2_p_kp) 
c
      b211A = 2.d0 * ( F2_2_kp_k * pklin_kp * pklin_k
     &     + F2_1_p_k  * pklin_p * pklin_k
     &     + F2_1_p_kp * pklin_p * pklin_kp ) * dd2**2
c
      b222A = 2.d0 * ( F2_2_kp_k * pklin_kp * pklin_k
     &     + F2_2_p_k  * pklin_p * pklin_k
     &     + F2_2_p_kp * pklin_p * pklin_kp ) * dd2**2
c
      b211tA = 2.d0 * (  F2_2_p_k * pklin_p * pklin_k
     &     + F2_1_kp_k * pklin_kp * pklin_k
     &     + F2_1_p_kp * pklin_kp * pklin_p ) * dd2**2
c
      b222tA = 2.d0 * (  F2_2_p_k * pklin_p * pklin_k
     &     + F2_2_kp_k * pklin_kp * pklin_k
     &     + F2_2_p_kp * pklin_kp * pklin_p ) * dd2**2
c
      end
c
c ******************************************************* c
c
      subroutine Bispec2(zred, p, kp, k, b221A, b212A, b221hA, b212hA)
c                                                         c
c ******************************************************* c
c
c     bispectrum at tree-level SPT (kp = k-p)
c
c      b221A = Bispec(2, 2, 1, p, kp, -k, zred)
c      b212A = Bispec(2, 1, 2, p, kp, -k, zred)
c
c      b221hA = Bispec(2, 2, 1, kp, -k, p, zred)
c      b212hA = Bispec(2, 1, 2, kp, -k, p, zred)
c
      implicit none
c
      real*8  zred, k, p, kp
      real*8  b221A, b212A, b221hA, b212hA
      real*8  dd2, growth, pklin_k, pklin_p, pklin_kp
      real*8  F2_1_kp_k, F2_2_kp_k, F2_1_p_k
      real*8  F2_2_p_k, F2_1_p_kp, F2_2_p_kp
c     -----------------------------------------   c
c
      dd2 = growth(zred)**2
c
      call find_pk(k, pklin_k)
      call find_pk(p, pklin_p)
      call find_pk(kp, pklin_kp)
c
c     bispectrum at tree-level order
c
      call F2_tree(k, p, kp, 
     &     F2_1_kp_k, F2_1_p_k, F2_1_p_kp, 
     &     F2_2_kp_k, F2_2_p_k, F2_2_p_kp)
c
      b221A = 2.d0 * ( F2_2_kp_k * pklin_kp * pklin_k
     &     + F2_2_p_k * pklin_p * pklin_k
     &     + F2_1_p_kp * pklin_p * pklin_kp ) * dd2**2
c
      b212A = 2.d0 * ( F2_2_kp_k * pklin_kp * pklin_k
     &     + F2_1_p_k * pklin_p * pklin_k
     &     + F2_2_p_kp * pklin_p * pklin_kp ) * dd2**2
c
      b221hA = 2.d0 * ( F2_2_p_k * pklin_p * pklin_k
     &     + F2_2_p_kp * pklin_p * pklin_kp 
     &     + F2_1_kp_k * pklin_kp * pklin_k ) * dd2**2
c
      b212hA = 2.d0 * ( F2_2_p_k * pklin_p * pklin_k
     &     + F2_1_p_kp * pklin_p * pklin_kp 
     &     + F2_2_kp_k * pklin_kp * pklin_k ) * dd2**2
c
      end
c
c ******************************************************* c
c
      subroutine F2_tree(k, p, kp, 
     &     F2_1_kp_k, F2_1_p_k, F2_1_p_kp, 
     &     F2_2_kp_k, F2_2_p_k, F2_2_p_kp) 
c
c ******************************************************* c
c
      implicit none
c
      real*8  k, p, kp, F2
      real*8  F2_1_kp_k, F2_1_p_k, F2_1_p_kp
      real*8  F2_2_kp_k, F2_2_p_k, F2_2_p_kp
c     -------------------------- 
c
         F2_1_kp_k = F2(1, kp, k, p) 
         F2_2_kp_k = F2(2, kp, k, p) 
         F2_1_p_k  = F2(1, p, k, kp) 
         F2_2_p_k  = F2(2, p, k, kp) 
         F2_1_p_kp = F2(1, p, kp, k) 
         F2_2_p_kp = F2(2, p, kp, k) 
c     
      end
c
c ******************************************************* c
c
      function F2(a, k1, k2, k3)
c
c ******************************************************* c
c
c     2nd-order PT kernel
c
c     Here we assume vec(k1) + vec(k2) + vec(k3) = 0 
c
      implicit none
      integer a
      real*8  F2, k1, k2, k3, k1dk2
c     --------------------------------------------------------
c
      k1dk2 = ( k3**2 - k1**2 - k2**2 ) / 2.d0
c
      if(a.eq.1) 
     &     F2 = 5.d0/7.d0 + 0.5d0*k1dk2*
     &     (1.d0/k1**2 + 1.d0/k2**2) + 2.d0/7.d0*(k1dk2/(k1*k2))**2
      if(a.eq.2) 
     &     F2 = 3.d0/7.d0 + 0.5d0*k1dk2*
     &     (1.d0/k1**2 + 1.d0/k2**2) + 4.d0/7.d0*(k1dk2/(k1*k2))**2
c
      end
c
c ******************************************************* c
c
      function growth(zred)
c
c ******************************************************* c
c
c     Linear growth factor for flat cosmology
c
      implicit none
      real*8 growth, zred, Omega_m, Omega_v, w_de, sigma8, a, b, c
      real*8 zred1, zi, zz, gz, g0
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
c     --------------------------------------------
c
      a = -1.d0 / (3.d0 * w_de)
      b = (w_de - 1.d0)/ (2.d0 * w_de)
      c = 1.d0 - 5.d0 / (6.d0 * w_de)
c
      zred1 = 1.d0 + zred
      zi = - Omega_v / Omega_m
      zz = zi * zred1**(3.d0*w_de) 
c
      call HYGFX(a,b,c,zi, g0)  
      call HYGFX(a,b,c,zz, gz)  
c
      growth = (gz/g0) / zred1
c
      end
c
c ******************************************************* c
c
      function f(zred)
c
c ******************************************************* c
c
c     d lnD_+(z) / d ln a,   as function of redshift 
c
      implicit none
c
      real*8  f, zred, zred1, Omega_m, Omega_v, w_de, sigma8
      real*8  a, b, c, zi, zz, g1, g2
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
c     ---------------------------------------------------
c
      zred1 = 1.d0 + zred
      zi = - Omega_v / Omega_m
      zz = zi * zred1**(3.d0*w_de) 
c
      a = 1.d0 - 1.d0 / (3.d0 * w_de)
      b = 1.5d0 - 1.d0 / (2.d0 * w_de)
      c = 2.d0 - 5.d0 / (6.d0 * w_de)
      call HYGFX(a,b,c,zz, g1)  
c
      a = - 1.d0 / (3.d0 * w_de)
      b = (-1.d0 + w_de) / (2.d0 * w_de)
      c = 1.d0 - 5.d0 / (6.d0 * w_de)
      call HYGFX(a,b,c,zz, g2)  
c
      f = 1.d0 + 3.d0 * (-1.d0 + w_de) / (6.d0*w_de -5.d0) 
     &     * zz * g1 / g2
c
      end
c
c ******************************************************* c
c
        SUBROUTINE HYGFX(A,B,C,X,HF)
c
c ******************************************************* c
C
C       ====================================================
C       Purpose: Compute hypergeometric function F(a,b,c,x)
C       Input :  a --- Parameter
C                b --- Parameter
C                c --- Parameter, c <> 0,-1,-2,...
C                x --- Argument   ( x < 1 )
C       Output:  HF --- F(a,b,c,x)
C       Routines called:
C            (1) GAMMAX for computing gamma function
C            (2) PSI for computing psi function
C       ====================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        LOGICAL L0,L1,L2,L3,L4,L5
        PI=3.141592653589793D0
        EL=.5772156649015329D0
        L0=C.EQ.INT(C).AND.C.LT.0.0
        L1=1.0D0-X.LT.1.0D-15.AND.C-A-B.LE.0.0
        L2=A.EQ.INT(A).AND.A.LT.0.0
        L3=B.EQ.INT(B).AND.B.LT.0.0
        L4=C-A.EQ.INT(C-A).AND.C-A.LE.0.0
        L5=C-B.EQ.INT(C-B).AND.C-B.LE.0.0
        IF (L0.OR.L1) THEN
           WRITE(*,*)'The hypergeometric series is divergent'
           RETURN
        ENDIF
        EPS=1.0D-15
        IF (X.GT.0.95) EPS=1.0D-8
        IF (X.EQ.0.0.OR.A.EQ.0.0.OR.B.EQ.0.0) THEN
           HF=1.0D0
           RETURN
        ELSE IF (1.0D0-X.EQ.EPS.AND.C-A-B.GT.0.0) THEN
           CALL GAMMAX(C,GC)
           CALL GAMMAX(C-A-B,GCAB)
           CALL GAMMAX(C-A,GCA)
           CALL GAMMAX(C-B,GCB)
           HF=GC*GCAB/(GCA*GCB)
           RETURN
        ELSE IF (1.0D0+X.LE.EPS.AND.DABS(C-A+B-1.0).LE.EPS) THEN
           G0=DSQRT(PI)*2.0D0**(-A)
           CALL GAMMAX(C,G1)
           CALL GAMMAX(1.0D0+A/2.0-B,G2)
           CALL GAMMAX(0.5D0+0.5*A,G3)
           HF=G0*G1/(G2*G3)
           RETURN
        ELSE IF (L2.OR.L3) THEN
           IF (L2) NM=INT(ABS(A))
           IF (L3) NM=INT(ABS(B))
           HF=1.0D0
           R=1.0D0
           DO 10 K=1,NM
              R=R*(A+K-1.0D0)*(B+K-1.0D0)/(K*(C+K-1.0D0))*X
10            HF=HF+R
           RETURN
        ELSE IF (L4.OR.L5) THEN
           IF (L4) NM=INT(ABS(C-A))
           IF (L5) NM=INT(ABS(C-B))
           HF=1.0D0
           R=1.0D0
           DO 15 K=1,NM
              R=R*(C-A+K-1.0D0)*(C-B+K-1.0D0)/(K*(C+K-1.0D0))*X
15            HF=HF+R
           HF=(1.0D0-X)**(C-A-B)*HF
           RETURN
        ENDIF
        AA=A
        BB=B
        X1=X
        IF (X.LT.0.0D0) THEN
           X=X/(X-1.0D0)
           IF (C.GT.A.AND.B.LT.A.AND.B.GT.0.0) THEN
              A=BB
              B=AA
           ENDIF
           B=C-B
        ENDIF
        IF (X.GE.0.75D0) THEN
           GM=0.0D0
           IF (DABS(C-A-B-INT(C-A-B)).LT.1.0D-15) THEN
              M=INT(C-A-B)
              CALL GAMMAX(A,GA)
              CALL GAMMAX(B,GB)
              CALL GAMMAX(C,GC)
              CALL GAMMAX(A+M,GAM)
              CALL GAMMAX(B+M,GBM)
              CALL PSI(A,PA)
              CALL PSI(B,PB)
              IF (M.NE.0) GM=1.0D0
              DO 30 J=1,ABS(M)-1
30               GM=GM*J
              RM=1.0D0
              DO 35 J=1,ABS(M)
35               RM=RM*J
              F0=1.0D0
              R0=1.0D0
              R1=1.0D0
              SP0=0.D0
              SP=0.0D0
              IF (M.GE.0) THEN
                 C0=GM*GC/(GAM*GBM)
                 C1=-GC*(X-1.0D0)**M/(GA*GB*RM)
                 DO 40 K=1,M-1
                    R0=R0*(A+K-1.0D0)*(B+K-1.0)/(K*(K-M))*(1.0-X)
40                  F0=F0+R0
                 DO 45 K=1,M
45                  SP0=SP0+1.0D0/(A+K-1.0)+1.0/(B+K-1.0)-1.0/K
                 F1=PA+PB+SP0+2.0D0*EL+DLOG(1.0D0-X)
                 DO 55 K=1,250
                    SP=SP+(1.0D0-A)/(K*(A+K-1.0))+(1.0-B)/(K*(B+K-1.0))
                    SM=0.0D0
                    DO 50 J=1,M
50                     SM=SM+(1.0D0-A)/((J+K)*(A+J+K-1.0))+1.0/
     &                    (B+J+K-1.0)
                    RP=PA+PB+2.0D0*EL+SP+SM+DLOG(1.0D0-X)
                    R1=R1*(A+M+K-1.0D0)*(B+M+K-1.0)/(K*(M+K))*(1.0-X)
                    F1=F1+R1*RP
                    IF (DABS(F1-HW).LT.DABS(F1)*EPS) GO TO 60
55                  HW=F1
60               HF=F0*C0+F1*C1
              ELSE IF (M.LT.0) THEN
                 M=-M
                 C0=GM*GC/(GA*GB*(1.0D0-X)**M)
                 C1=-(-1)**M*GC/(GAM*GBM*RM)
                 DO 65 K=1,M-1
                    R0=R0*(A-M+K-1.0D0)*(B-M+K-1.0)/(K*(K-M))*(1.0-X)
65                  F0=F0+R0
                 DO 70 K=1,M
70                  SP0=SP0+1.0D0/K
                 F1=PA+PB-SP0+2.0D0*EL+DLOG(1.0D0-X)
                 DO 80 K=1,250
                    SP=SP+(1.0D0-A)/(K*(A+K-1.0))+(1.0-B)/(K*(B+K-1.0))
                    SM=0.0D0
                    DO 75 J=1,M
75                     SM=SM+1.0D0/(J+K)
                    RP=PA+PB+2.0D0*EL+SP-SM+DLOG(1.0D0-X)
                    R1=R1*(A+K-1.0D0)*(B+K-1.0)/(K*(M+K))*(1.0-X)
                    F1=F1+R1*RP
                    IF (DABS(F1-HW).LT.DABS(F1)*EPS) GO TO 85
80                  HW=F1
85               HF=F0*C0+F1*C1
              ENDIF
           ELSE
              CALL GAMMAX(A,GA)
              CALL GAMMAX(B,GB)
              CALL GAMMAX(C,GC)
              CALL GAMMAX(C-A,GCA)
              CALL GAMMAX(C-B,GCB)
              CALL GAMMAX(C-A-B,GCAB)
              CALL GAMMAX(A+B-C,GABC)
              C0=GC*GCAB/(GCA*GCB)
              C1=GC*GABC/(GA*GB)*(1.0D0-X)**(C-A-B)
              HF=0.0D0
              R0=C0
              R1=C1
              DO 90 K=1,250
                 R0=R0*(A+K-1.0D0)*(B+K-1.0)/(K*(A+B-C+K))*(1.0-X)
                 R1=R1*(C-A+K-1.0D0)*(C-B+K-1.0)/(K*(C-A-B+K))
     &              *(1.0-X)
                 HF=HF+R0+R1
                 IF (DABS(HF-HW).LT.DABS(HF)*EPS) GO TO 95
90               HW=HF
95            HF=HF+C0+C1
           ENDIF
        ELSE
           A0=1.0D0
           IF (C.GT.A.AND.C.LT.2.0D0*A.AND.
     &         C.GT.B.AND.C.LT.2.0D0*B) THEN
              A0=(1.0D0-X)**(C-A-B)
              A=C-A
              B=C-B
           ENDIF
           HF=1.0D0
           R=1.0D0
           DO 100 K=1,250
              R=R*(A+K-1.0D0)*(B+K-1.0D0)/(K*(C+K-1.0D0))*X
              HF=HF+R
              IF (DABS(HF-HW).LE.DABS(HF)*EPS) GO TO 105
100           HW=HF
105        HF=A0*HF
        ENDIF
        IF (X1.LT.0.0D0) THEN
           X=X1
           C0=1.0D0/(1.0D0-X)**AA
           HF=C0*HF
        ENDIF
        A=AA
        B=BB
        IF (K.GT.120) WRITE(*,115)
115     FORMAT(1X,'Warning! You should check the accuracy')
        RETURN
        END
c
c ******************************************************* c
c
        SUBROUTINE GAMMAX(X,GA)
c
c ******************************************************* c
C
C       ==================================================
C       Purpose: Compute gamma function â(x)
C       Input :  x  --- Argument of â(x)
C                       ( x is not equal to 0,-1,-2,úúú)
C       Output:  GA --- â(x)
C       ==================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION G(26)
        PI=3.141592653589793D0
        IF (X.EQ.INT(X)) THEN
           IF (X.GT.0.0D0) THEN
              GA=1.0D0
              M1=X-1
              DO 10 K=2,M1
10               GA=GA*K
           ELSE
              GA=1.0D+300
           ENDIF
        ELSE
           IF (DABS(X).GT.1.0D0) THEN
              Z=DABS(X)
              M=INT(Z)
              R=1.0D0
              DO 15 K=1,M
15               R=R*(Z-K)
              Z=Z-M
           ELSE
              Z=X
           ENDIF
           DATA G/1.0D0,0.5772156649015329D0,
     &          -0.6558780715202538D0, -0.420026350340952D-1,
     &          0.1665386113822915D0,-.421977345555443D-1,
     &          -.96219715278770D-2, .72189432466630D-2,
     &          -.11651675918591D-2, -.2152416741149D-3,
     &          .1280502823882D-3, -.201348547807D-4,
     &          -.12504934821D-5, .11330272320D-5,
     &          -.2056338417D-6, .61160950D-8,
     &          .50020075D-8, -.11812746D-8,
     &          .1043427D-9, .77823D-11,
     &          -.36968D-11, .51D-12,
     &          -.206D-13, -.54D-14, .14D-14, .1D-15/
           GR=G(26)
           DO 20 K=25,1,-1
20            GR=GR*Z+G(K)
           GA=1.0D0/(GR*Z)
           IF (DABS(X).GT.1.0D0) THEN
              GA=GA*R
              IF (X.LT.0.0D0) GA=-PI/(X*GA*DSIN(PI*X))
           ENDIF
        ENDIF
        RETURN
        END
c
c ******************************************************* c
c
        SUBROUTINE PSI(X,PS)
c
c ******************************************************* c
C
C       ======================================
C       Purpose: Compute Psi function
C       Input :  x  --- Argument of psi(x)
C       Output:  PS --- psi(x)
C       ======================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        XA=DABS(X)
        PI=3.141592653589793D0
        EL=.5772156649015329D0
        S=0.0D0
        IF (X.EQ.INT(X).AND.X.LE.0.0) THEN
           PS=1.0D+300
           RETURN
        ELSE IF (XA.EQ.INT(XA)) THEN
           N=XA
           DO 10 K=1 ,N-1
10            S=S+1.0D0/K
           PS=-EL+S
        ELSE IF (XA+.5.EQ.INT(XA+.5)) THEN
           N=XA-.5
           DO 20 K=1,N
20            S=S+1.0/(2.0D0*K-1.0D0)
           PS=-EL+2.0D0*S-1.386294361119891D0
        ELSE
           IF (XA.LT.10.0) THEN
              N=10-INT(XA)
              DO 30 K=0,N-1
30               S=S+1.0D0/(XA+K)
              XA=XA+N
           ENDIF
           X2=1.0D0/(XA*XA)
           A1=-.8333333333333D-01
           A2=.83333333333333333D-02
           A3=-.39682539682539683D-02
           A4=.41666666666666667D-02
           A5=-.75757575757575758D-02
           A6=.21092796092796093D-01
           A7=-.83333333333333333D-01
           A8=.4432598039215686D0
           PS=DLOG(XA)-.5D0/XA+X2*(((((((A8*X2+A7)*X2+
     &        A6)*X2+A5)*X2+A4)*X2+A3)*X2+A2)*X2+A1)
           PS=PS-S
        ENDIF
        IF (X.LT.0.0) PS=PS-PI*DCOS(PI*X)/DSIN(PI*X)-1.0D0/X
        RETURN
        END
c
c ************************************************************
c
      SUBROUTINE polint(xa,ya,n,x,y,dy)
c
c ************************************************************
c
      implicit none
      INTEGER n,NMAX
      REAL*8 dy,x,y,xa(n),ya(n)
      PARAMETER (NMAX=10)
      INTEGER i,m,ns
      REAL*8 den,dif,dift,ho,hp,w,c(NMAX),d(NMAX)
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.)pause 'failure in polint'
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      return
      END
C
c ************************************************ c
c
      SUBROUTINE hunt(xx,n,x,jlo)
c
c ************************************************ c
c
      implicit none
      INTEGER jlo,n
      REAL*8 x,xx(n)
      INTEGER inc,jhi,jm
      LOGICAL ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        goto 3
      endif
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          goto 1
        endif
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          goto 2
        endif
      endif
3     if(jhi-jlo.eq.1)return
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      endif
      goto 3
      END
c
c ************************************************************
c
      SUBROUTINE gauleg(x1,x2,x,w,n)
c
c ************************************************************
c
      INTEGER n
      REAL*8 x1,x2,x(n),w(n)
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.d-14)
      INTEGER i,j,m
      DOUBLE PRECISION p1,p2,p3,pp,xl,xm,z,z1
      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      do 12 i=1,m
        z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
11        continue
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.EPS)goto 1
        x(i)=xm-xl*z
        x(n+1-i)=xm+xl*z
        w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
        w(n+1-i)=w(i)
12    continue
      return
      END
